@@ -6,6 +6,7 @@ del get_versions
 
 from .annotation import Annotation, AnnotationDet
 from .configurable import Configurable
+from .predictor import Predictor
 from . import datasets
 from . import decoder
 from . import encoder
@@ -15,10 +16,20 @@ from . import metric
 from . import network
 from . import optimize
 from . import plugin
+from . import visualizer
 
 from .datasets import DATAMODULES
 from .decoder import DECODERS
-from .network.factory import BASE_FACTORIES, BASE_TYPES, CHECKPOINT_URLS, HEADS
+from .network.factory import (
+    BASE_FACTORIES,
+    BASE_TYPES,
+    CHECKPOINT_URLS,
+    HEADS,
+    PRETRAINED_UNAVAILABLE,
+)
 from .network.losses.factory import LOSSES, LOSS_COMPONENTS
 from .network.nets import MODEL_MIGRATION
 from .show.annotation_painter import PAINTERS
+
+# load plugins last
+plugin.register()
