@@ -19,10 +19,10 @@ def main():
 
     parser.add_argument('--debug', default=False, action='store_true')
     parser.add_argument('--output')
-    network.cli(parser)
+    network.Factory.cli(parser)
     args = parser.parse_args()
 
-    network.configure(args)
+    network.Factory.configure(args)
 
     if args.checkpoint is None:
         raise Exception('checkpoint must be provided for old model to migrate from')
@@ -37,7 +37,7 @@ def main():
 
     # create a new model
     args.checkpoint = None
-    new_model, _ = network.factory_from_args(args)
+    new_model, _ = network.Factory().factory()
 
     # transfer state from old to new model
     new_model.load_state_dict(checkpoint['model'].state_dict())
