@@ -3,11 +3,15 @@ import logging
 
 import numpy as np
 import PIL
-import scipy
 import torch
 
 from .preprocess import Preprocess
 
+try:
+    import scipy
+except ImportError:
+    scipy = None
+
 LOG = logging.getLogger(__name__)
 
 
