@@ -1,9 +1,12 @@
 import logging
 
-import scipy
-
 from .base import Base
 
+try:
+    import scipy
+except ImportError:
+    scipy = None
+
 LOG = logging.getLogger(__name__)
 
 
@@ -22,7 +25,7 @@ class Occupancy(Base):
             # occupancy maps are at a reduced scale wrt the processed image
             # pylint: disable=unsubscriptable-object
             factor = 1.0 / occupancy.reduction
-            reduced_image = scipy.ndimage.zoom(self._processed_image, (factor, factor, 1), order=1)
+            reduced_image = scipy.ndimage.zoom(self.processed_image(), (factor, factor, 1), order=1)
 
             with self.image_canvas(reduced_image) as ax:
                 occ = occupancy.occupancy[f].copy()
